<?php
/**
* This file contains the code for querying eXist XML:DB via SOAP interface.
*
* PHP version 5
*
* @author     Anton Kohutovic, Oscar Celma - base
* @license    GPL
*/

/**
* exist Class
*
* This class is the main interface for making soap requests to eXist XML:DB.
*
* @access   public
*/
require_once "NXDapi.php";

class Exist
 implements NXDapi
{
  protected $wsdl     = "";
  protected $user     = "guest";
  protected $password = "guest";

  protected $soapClient = null;
  protected $session    = null;
  protected $error      = "";

  private $highlight = false;

  protected $result;
  protected $queryTime;
  protected $debug=true;

  public function __construct($user="guest", $password="guest", $wsdl="http://localhost:8080/exist/services/Query?wsdl")
  {
    $this->user = $user;
    $this->password = $password;
    $this->wsdl = $wsdl;
    try
    {
      @$this->soapClient = new SoapClient($this->wsdl);
    }
     catch(SoapFault $e)
    {
      $this->setError($e->faultstring,"fatal error");
      die($this->getError());
    }
  }


  public function disconnect()
  {
    if ( $this->getError() )
      return false;
    try
    {
      $parameters = array('sessionId' => $this->session);
      $this->session = $this->soapCall('disconnect', $parameters);
    }
    catch(SoapFault $e)
    {
      $this->setError($e->faultstring);
    }
  }

  public function connect()
  {
    if ($this->getError())
      return false;
    try
    {
      $parameters = array('userId' => $this->user, 'password' => $this->password);
      $this->session = $this->soapCall('connect', $parameters);
    }
    catch(SoapFault $e)
    {
      $this->setError($e->faultstring,"fatal error");
      return false;
    }
    return true;
  }

  public function getError()
  {
   
    if ($this->error != "")
    {
      $fp = fopen("exist.log","a");
      fputs($fp,date("Y/m/d - H:i:s",time()).' '.$this->error."\n");
      fclose($fp);
      redirect("working.php");
      return $this->error;
    }
    return false;
  }

  public function execute($query)
  {
    // $query = htmlentities($query);
    if (empty($query))
    {
      $this->error = "ERROR: Query is empty!";
      return false;
    }
    try
    {
      if (!version_compare(PHP_VERSION, '5.1.0', 'ge'))
      {
        $query = base64_encode($query);
      }
      $parameters = array('sessionId' => $this->session , 'xquery' => $query );
      $queryResponse = $this->soapCall('xquery', $parameters);
    }
    catch(SoapFault $e)
    {
      $this->setError($e->faultstring);
      return false;
    }

    if (is_object($queryResponse) && $queryResponse->hits > 0)
    {
      $parameters = array('sessionId' => $this->session ,
                    'start' => 1,
                    'howmany' => $queryResponse->hits,
                    'indent' => TRUE,
                    'xinclude' => TRUE,
                    'highlight' => $this->highlight
                    );
      $xml = $this->soapCall('retrieve', $parameters);

    }
    else
    {
      if (is_object($queryResponse))
      {
        $this->result=Null;
        return true;
      }
      else $this->setError('No data found!');

      return false;
    }

    $this->result    = $xml;
    $this->queryTime = $queryResponse->queryTime;
    $this->hits      = $queryResponse->hits;
    $this->collection= $queryResponse->collections;
    return true;
  }

  protected function soapCall($function, $params)
  {
    $return = $this->soapClient->__soapCall($function, array('parameters'=>$params));
    $output = $function . "Return";
    return property_exists($return, $output) ? $return->$output : 0;
  }

  public function setHighlight($highlight)
  {
    $this->highlight = $highlight ? 'both' : FALSE;
  }

  public function setDebug($debug=true)
  {
    $this->debug = $debug;
  }

  public function setUser($user)
  {
    $this->user = $user;
  }

  public function setPassword($passwd)
  {
    $this->password = $passwd;
  }

  public function setWSDL($wsdl)
  {
    $this->wsdl = $wsdl;
  }

  protected function setError($error,$level="error")
  {
    if ($this->debug) $this->error = "<div style=\"color:red;\">eXist ".$level.": ".$error."<div>";
    else $this->error = "eXist ".$level.": ".$error;
  }

  public function fetchNode($query)
  {
    if (!$this->execute($query)) 
    {
      $this->setError($query.$this->error, 'Query:');
      die($this->getError());
    }
    if (is_array($this->result)) return implode("\n",$this->result);
    else return $this->result;
  }

  public function getQueryTime($formatted=false)
  {
    if ($formatted) return ($this->queryTime/1000)."s";
    else  return $this->queryTime;
  }

  public function fetchArray($query)
  {
     $this->execute($query) or die(dumpxml($query)."<br />".$this->getError());
     return $this->result;
  }

}

/**
 * eXistAdmin Class
 *
 * This class is the main interface for manipulating the collections and documents into the eXist XML:DB.
 *
 * basic usage:<code>
 * include "existAdmin.php";
 * $db = new eXistAdmin('guest', 'guest', 'http://127.0.0.1:8080/exist/services/Admin?wsdl');
 * $db->connect() or die ($db->getError());
 *
 * // Store Document
 * echo $db->store('<simple><fxp>fran�ois</fxp></simple>',
 *                              'UTF-8',
 *                              '/db/test.xml', true);
 * echo $db->store('<simple><fxp>fran�ois</fxp></simple>',
 *                              'UTF-8',
 *                              '/db/test2suppr.xml', true);
 * // Remove Document
 * echo $db->removeDocument('/db/test2suppr.xml');
 *
 * // Create collection
 * echo $db->createCollection('/db/existAdminDemo');
 * echo $db->createCollection('/db/existAdminDemo2supp');
 *
 * // Remove collection
 * echo $db->removeCollection('/db/existAdminDemo2supp');
 *
 * // XupdateResource
 * $xupdate = "<xupdate:modifications version='1.0' xmlns:xupdate='http://www.xmldb.org/xupdate'>".
 *              "<xupdate:update select='/simple/fxp'>TITI</xupdate:update></xupdate:modifications>";
 *
 * echo $db->xupdateResource('/db/test.xml', $xupdate);
 *
 * $db->disconnect() or die ($db->getError());
 * </code>
 *
 * @access   public
 */

class eXistAdmin extends Exist
{
  public function __construct($user="guest", $password="guest", $wsdl="http://localhost:8080/exist/services/Admin?wsdl")
  {
    $this->user = $user;
    $this->password = $password;
    $this->wsdl = $wsdl;

    $this->soapClient = new SoapClient ($this->wsdl);
    //dump($this->soapClient);
  }

  public function __destruct() {
  }

/*
 * Store
 *    <element name="store">
 *    <complexType>
 *     <sequence>
 *      <element name="sessionId" type="xsd:string"/>
 *      <element name="data" type="xsd:base64Binary"/>
 *      <element name="encoding" type="xsd:string"/>
 *      <element name="path" type="xsd:string"/>
 *      <element name="replace" type="xsd:boolean"/>
 *     </sequence>
 *    </complexType>
 *   </element>
 */
    public function store($data, $encoding = "UTF-8", $path = "/db", $replace = false)
{
  if ( $this->getError() )
    return false;
  if ( empty($data) )
  {
    $this->error = "ERROR: No data to load !";
    return false;
  }
  try
  {
                  if (!version_compare(PHP_VERSION, '5.1.0', 'ge')) {
                    $data = base64_encode($data);
                      }
                  $parameters = array('sessionId' => $this->session ,
                                      'data' => $data,
                                      'encoding' => $encoding,
                                      'path' => $path,
                                      'replace' => $replace );

                  $queryResponse = $this->soapCall('store', $parameters);

  }
  catch( SoapFault $e )
  {
    $this->setError($e->faultstring);
    return false;
  }

  if ( $this->debug && is_object($queryResponse) )
  {
                  // xquery call Result
                      print "===========================================================================";
                  print "<p><b>Result of the <i>store</i> SOAP call (in PHP array format)</b></p>";
                  print "===========================================================================";
                  print "<p>\$queryResponse:<p><pre>";
                  print_r($queryResponse);
                  print "</pre>";
                  print "===========================================================================";
  }

  return true;
}

/*
 * createCollection
 *
 *  <element name="createCollection">
 *    <complexType>
 *     <sequence>
 *      <element name="sessionId" type="xsd:string"/>
 *      <element name="path" type="xsd:string"/>
 *     </sequence>
 *   </complexType>
 */
    public function createCollection($path)
{
  if ( $this->getError() )
    return false;
  if ( empty($path) )
  {
    $this->error = "ERROR: path is empty!";
    return false;
  }
  try
  {
                  // encode only to base64 if php version lesser than 5.1
                  // patch by Bastian Gorke bg/at\ipunkt/dot\biz
                      if (!version_compare(PHP_VERSION, '5.1.0', 'ge')) {
                    $xupdate = base64_encode($xupdate);
                      }
                  //$queryResponse = $this->_soapClient->xquery($this->session, $queryBase64);
                  $parameters = array('sessionId' => $this->session , 'path' => $path);

                  $queryResponse = $this->soapCall('createCollection', $parameters);
  }
  catch( SoapFault $e )
  {
    $this->setError($e->faultstring);
    return false;
  }

  if ( $this->debug && is_object($queryResponse) )
  {
                  // xquery call Result
                      print "===========================================================================";
                  print "<p><b>Result of the <i>store</i> SOAP call (in PHP array format)</b></p>";
                  print "===========================================================================";
                  print "<p>\$queryResponse:<p><pre>";
                  print_r($queryResponse);
                  print "</pre>";
                  print "===========================================================================";
  }


  return $queryResponse->createCollectionReturn;
}

/*
 * removeCollection
 *
 *  <element name="removeCollection">
 *    <complexType>
 *     <sequence>
 *      <element name="sessionId" type="xsd:string"/>
 *      <element name="path" type="xsd:string"/>
 *     </sequence>
 *   </complexType>
 */
    public function removeCollection($path)
{
  if ( $this->getError() )
    return false;
  if ( empty($path) )
  {
    $this->error = "ERROR: path is empty!";
    return false;
  }
  try
  {
                  // encode only to base64 if php version lesser than 5.1
                  // patch by Bastian Gorke bg/at\ipunkt/dot\biz
                      if (!version_compare(PHP_VERSION, '5.1.0', 'ge')) {
                    $xupdate = base64_encode($xupdate);
                      }
                  //$queryResponse = $this->_soapClient->xquery($this->session, $queryBase64);
                  $parameters = array('sessionId' => $this->session , 'path' => $path);

                  $queryResponse = $this->soapCall('removeCollection', $parameters);
  }
  catch( SoapFault $e )
  {
    $this->setError($e->faultstring);
    return false;
  }


  if ( $this->debug && is_object($queryResponse) )
  {
                  // xquery call Result
                      print "===========================================================================";
                  print "<p><b>Result of the <i>store</i> SOAP call (in PHP array format)</b></p>";
                  print "===========================================================================";
                  print "<p>\$queryResponse:<p><pre>";
                  print_r($queryResponse);
                  print "</pre>";
                  print "===========================================================================";
  }

  return $queryResponse->removeCollectionReturn;
}

/*
 * removeDocument
 *
 *  <element name="removeDocument">
 *    <complexType>
 *     <sequence>
 *      <element name="sessionId" type="xsd:string"/>
 *      <element name="path" type="xsd:string"/>
 *     </sequence>
 *   </complexType>
 */
    public function removeDocument($path)
{
  if ( $this->getError() )
    return false;
  if ( empty($path) )
  {
    $this->error = "ERROR: path is empty!";
    return false;
  }
  try
  {
                  // encode only to base64 if php version lesser than 5.1
                  // patch by Bastian Gorke bg/at\ipunkt/dot\biz
                      if (!version_compare(PHP_VERSION, '5.1.0', 'ge')) {
                    $xupdate = base64_encode($xupdate);
                      }
                      $parameters = array('sessionId' => $this->session , 'path' => $path);

                      $queryResponse = $this->soapCall('removeDocument', $parameters);
  }
  catch( SoapFault $e )
  {
    $this->setError($e->faultstring);
    return false;
  }


  if ( $this->debug && is_object($queryResponse) )
  {
                  // xquery call Result
                      print "===========================================================================";
                  print "<p><b>Result of the <i>store</i> SOAP call (in PHP array format)</b></p>";
                  print "===========================================================================";
                  print "<p>\$queryResponse:<p><pre>";
                  print_r($queryResponse);
                  print "</pre>";
                  print "===========================================================================";
  }

  return $queryResponse->removeDocumentReturn;
}

/*
 * xupdateResource
 *
 *   <element name="xupdate">
 *    <complexType>
 *     <sequence>
 *      <element name="sessionId" type="xsd:string"/>
 *      <element name="collectionName" type="xsd:string"/>
 *      <element name="xupdate" type="xsd:string"/>
 *     </sequence>
 *    </complexType>
 */
    public function xupdate($collectionName, $xupdate)
{
  if ( $this->getError() )
    return false;
  if ( empty($xupdate) )
  {
    $this->error = "ERROR: Xupdate query is empty!";
    return false;
  }
  try
  {
                  // encode only to base64 if php version lesser than 5.1
                  // patch by Bastian Gorke bg/at\ipunkt/dot\biz
                      if (!version_compare(PHP_VERSION, '5.1.0', 'ge')) {
                    $xupdate = base64_encode($xupdate);
                      }
                  //$queryResponse = $this->_soapClient->xquery($this->session, $queryBase64);
                  $parameters = array('sessionId' => $this->session , 'collectionName' => $collectionName, 'xupdate' => $xupdate );

                  $queryResponse = $this->soapCall('xupdate', $parameters);
  }
  catch( SoapFault $e )
  {
    $this->setError($e->faultstring);
    return false;
  }

  if ( $this->debug && is_object($queryResponse) )
  {
                  // xquery call Result
                      print "===========================================================================";
                  print "<p><b>Result of the <i>store</i> SOAP call (in PHP array format)</b></p>";
                  print "===========================================================================";
                  print "<p>\$queryResponse:<p><pre>";
                  print_r($queryResponse);
                  print "</pre>";
                  print "===========================================================================";
  }

  return $queryResponse->xupdateReturn;
}

/*
 * xupdateResource
 *
 * <element name="xupdateResource">
 *   <complexType>
 *    <sequence>
 *     <element name="sessionId" type="xsd:string"/>
 *     <element name="documentName" type="xsd:string"/>
 *     <element name="xupdate" type="xsd:string"/>
 *    </sequence>
 *   </complexType>
 *   </element>
 */
    public function xupdateResource($documentName, $xupdate)
{
  if ( $this->getError() )
    return false;
  if ( empty($xupdate) )
  {
    $this->error = "ERROR: Xupdate query is empty!";
    return false;
  }
  try
  {
                  // encode only to base64 if php version lesser than 5.1
                  // patch by Bastian Gorke bg/at\ipunkt/dot\biz
                      if (!version_compare(PHP_VERSION, '5.1.0', 'ge')) {
                    $xupdate = base64_encode($xupdate);
                      }
                  //$queryResponse = $this->_soapClient->xquery($this->session, $queryBase64);
                  $parameters = array('sessionId' => $this->session , 'documentName' => $documentName, 'xupdate' => $xupdate );

                  $queryResponse = $this->soapCall('xupdateResource', $parameters);
  }
  catch( SoapFault $e )
  {
    $this->setError($e->faultstring);
    return false;
  }

  if ( $this->debug && is_object($queryResponse) )
  {
                  // xquery call Result
                      print "===========================================================================";
                  print "<p><b>Result of the <i>store</i> SOAP call (in PHP array format)</b></p>";
                  print "===========================================================================";
                  print "<p>\$queryResponse:<p><pre>";
                  print_r($queryResponse);
                  print "</pre>";
                  print "===========================================================================";
  }

  return $queryResponse->xupdateResourceReturn;
}
}

?>
