#include <vector>
#include <iostream>
#include <math.h>


using std::vector;
using std::ostream;
using std::cout;
using std::endl;

class Matrix
{
   //-public member data
   public:
     //-constructor, copy constructor, and deconstructor
     Matrix(const vector< vector<double> > &m);
     Matrix(const Matrix& m);
     ~Matrix();

     //-Matrix operations.
     double find_determinant();
     double L2norm();
     
     Matrix find_product(const Matrix& m) const;
     Matrix scalar_product(double k) const;
     Matrix find_difference(const Matrix& m) const;
     Matrix find_sum(const Matrix& m) const;
     Matrix find_inverse() const;
     Matrix transpose_matrix() const;

     //-getter methods
     int get_rows() const;
     int get_cols() const;
     double &value(int row, int col);

     //-overloaded operators
     friend ostream& operator<<(ostream& output_stream, const Matrix& print_matrix);
     Matrix operator+ (const Matrix& m) const;
     Matrix operator- (const Matrix& m) const;
     Matrix operator* (const Matrix& m) const;
     Matrix operator* (double k) const;
     Matrix operator/ (const Matrix& m) const;
     double operator() (int row, int col) const;

  //-private member data
  private:
     vector< vector<double> > mat;
     int columns;
     int rows;
};

/**Constructor*/
Matrix::Matrix(const vector< vector<double> > &m)
:mat(m),rows(m.size()),columns(m[0].size())
{
}
/**Copy Constructor*/
Matrix::Matrix(const Matrix& m)
{
}
/**Deconstructor*/
Matrix::~Matrix()
{
}

/**Overloaded insertion operator*/
ostream& operator<<(ostream& output_stream, const Matrix& print_matrix)
{
  cout << "--- Matrix ---" << endl;
  for (int r=0; r<print_matrix.get_rows(); ++r)
  {
    for (int c=0; c<print_matrix.get_cols(); ++c)
    {
       output_stream << print_matrix(r,c) << " ";
    }
    output_stream << endl;
  }
  cout << "---end---" << endl;
  return output_stream;
}
double Matrix::operator() (int row, int col) const
{
       return this->mat[row][col];
}

/**Overloaded addition operator*/
Matrix Matrix::operator+ (const Matrix& m) const
{
  return this->find_sum(m);
}

/**Overloaded subtraction operator*/
Matrix Matrix::operator- (const Matrix& m) const
{
  return this->find_difference(m);
}

/**Overloaded multiplication operator*/
Matrix Matrix::operator* (const Matrix& m) const
{
  return this->find_product(m);
}

/**Overloaded multiplication operator*/
Matrix Matrix::operator* (double k) const
{
  return this->scalar_product(k);
}

/**Overloaded division operator*/
Matrix Matrix::operator/ (const Matrix& m) const
{
  return ( this->find_product( (m.find_inverse()) ) );
}

/**get_rows() const
 *-RETURNS: number of rows contained in the matrix.
 */
int Matrix::get_rows() const
{
   return this->rows;
}

/**get_cols() const
 *-RETURNS: number of columns contained in the matrix.
 */
int Matrix::get_cols() const
{
   return this->columns;
}


/**find_determinant()
 *-RETURNS: determinant of the current Matrix object.
 *-USES: gauss-jordan elimination method.
 */
double Matrix::find_determinant()
{
  Matrix matrix(this->mat);
  Matrix new_matrix(this->mat);
  Matrix temp_matrix(this->mat);

  int SIZE = 0;

  if (matrix.get_rows() != matrix.get_cols())
  {
    return 0;
  }
  else
  {
    SIZE = matrix.get_rows();
  }

  if (SIZE == 2)
  {
    return ( ( matrix(0,0)*matrix(1,1) ) - ( matrix(0,1)*matrix(1,0) ) );
  }
  else
  {
    int pivot_row=0;
    int pivot_col=0;
    double pivot_point = matrix(pivot_row,pivot_col);

    bool last_pivot = false;
    bool first_pivot = true;
    while (!last_pivot)
    {
       for (int row=(pivot_row+1); row<SIZE; ++row)
       {
         for (int col=0; col<SIZE; ++col)
         {
             //Check if last pivot.
              if (pivot_point == 0)
              {
                  new_matrix.value(row,col) = 0;
              }
              else
              {
                  if (first_pivot != false)
                  {
                  
                      new_matrix.value(row,col) = matrix(row,col) - matrix(row,pivot_col) *
                                         matrix(pivot_row,col)/pivot_point;
                  }
                  else
                  {

                      new_matrix.value(row,col) = temp_matrix(row,col) - temp_matrix(row,pivot_col)*
                                             temp_matrix(pivot_row,col)/pivot_point;
                  }
              } 
          }
        }
        if (pivot_row == (SIZE-2))
        {
          last_pivot = true;
          break;
        }
        //Move the pivot up one
        pivot_row = pivot_row + 1;
        pivot_col = pivot_col + 1;
        pivot_point = new_matrix(pivot_row,pivot_col);
        first_pivot = false;
        temp_matrix = new_matrix;
    }
      
    //compute the determinant from the new matrix
    double determinant = 1;
    for (int r=0; r<SIZE; ++r)
    {
        determinant = determinant * new_matrix(r,r);
    }

    return determinant;
   }
   return 0;
}

/**find_product(matrix)
 *-RETURNS: newly formed matrix resulting from the product of two matrices.
 */
Matrix Matrix::find_product(const Matrix& m) const
{
   int rows = this->get_rows();
   int cols = this->get_cols();
   int new_matrix_cols = m.get_cols();
   vector<vector<double> > matrix(rows,
                                      vector<double>(new_matrix_cols, 0));
  vector<vector<double> > temp_2d = this->mat;
  Matrix temp(temp_2d);
   Matrix new_matrix(matrix);
   for (int i=0; i<rows; i++)
   {
     for (int j=0; j<new_matrix_cols; j++)
     {
        double element_value = 0;
        for (int k=0; k<cols; k++)
        {
          element_value += temp(i,k)*m.mat[k][j];
        }
        new_matrix.value(i,j) = element_value;
     }
   }
   return new_matrix;
}

Matrix Matrix::scalar_product(double k) const
{
  int new_matrix_rows = this->get_rows();
  int new_matrix_cols = this->get_cols();
  vector<vector<double> > matrix(new_matrix_rows,
                                 vector<double>(new_matrix_cols, 0));
  vector<vector<double> > temp_2d = this->mat;
  Matrix temp(temp_2d);
  Matrix new_matrix(matrix);
  for (int i=0; i<new_matrix_rows; i++)
  {
    for (int j=0; j<new_matrix_cols; j++)
    {
      new_matrix.value(i,j) = this->mat[i][j]*k;
      
    }
  }
  return new_matrix;
}
/**find_inverse()
 *-RETURNS: newly formed matrix which is the inverse of the current matrix
 *-PRE: matrix must not have a determinant of zero and must be a square matrix
 *-USES: adjoint method
 **/
Matrix Matrix::find_inverse() const
{
  int old_rows = this->get_rows();
  int old_cols = this->get_cols();
  
  vector< vector<double> > vect_matrix(old_rows, vector<double>(old_cols,0));
  Matrix new_matrix(vect_matrix);
  Matrix temp(this->mat);
  
  //if the determinant is zero or the matrix is not square,
  //-then it does not have an inverse.  since a matrix object
  //-must be returned, send back one filled with zeros (showing it 
  //-doesn't have an inverse)
  if ( (temp.find_determinant() == 0) || (old_rows != old_cols) )
  {
     return Matrix( vector< vector<double> >(old_rows, vector<double>(old_cols,0)) );                   
  }
  
  //check if two x two square matrix
  if (new_matrix.get_rows() == 2)
  {
     double temp_zero = temp(0,0);
     double temp_one = temp(0,1);
     double determinant = temp.find_determinant();
     //change left diagnol
     new_matrix.value(0,0) = temp(1,1);
     new_matrix.value(1,1) = temp_zero;
       
     //change right diagnol 
     new_matrix.value(0,1) = temp(1,0);
     new_matrix.value(1,0) = temp_one;
       
     for (int rows=0; rows<2; ++rows)
     {
        for (int cols=0; cols<2; ++cols)
        {
           new_matrix.value(rows,cols) = new_matrix(rows,cols)/determinant;
        }
     }  
       
  }
  else
  {
     //if matrix is larger than 2x2 use the adjoint method
     vector< vector<double> > cofactor_matrix(old_rows-1, vector<double>(old_cols-1,0));
     Matrix cofactor(cofactor_matrix);
     bool is_negative = false;
             //get cofactor matrix
             //-a cofactor matrix is the original matrix excluding
             //-the current elements row AND column.
             for (int cof_row=0; cof_row<old_rows; ++cof_row)
             {
                 for (int cof_col=0; cof_col<old_cols; ++cof_col)
                 {
                     
                     for (int r=0; r<old_rows; ++r)
                     {
                         for (int c=0; c<old_cols; ++c)
                         {
                             if ((cof_row != r) && (cof_col != c))
                             {
                                if ((r < cof_row) && (c < cof_col))
                                {
                                   cofactor.value(r,c) = temp(r,c);
                                }
                                else if ((r < cof_row) && (c > cof_col))
                                {
                                   cofactor.value(r,c-1) = temp(r,c);
                                }
                                else if ((r > cof_row) && (c < cof_col))
                                {
                                   cofactor.value(r-1,c) = temp(r,c);
                                }
                                else
                                {
                                   cofactor.value(r-1,c-1) = temp(r,c);
                                }
                             }
                         }
                     }
                     //the new value in the matrix to be returned is the 
                     //-determinant of the cofactor matrix.  +/- signs are
                     //-used interchangably
                     if (is_negative)
                     {
                        new_matrix.value(cof_row,cof_col) = -(cofactor.find_determinant());
                        is_negative = false;
                     }
                     else
                     {
                         new_matrix.value(cof_row,cof_col) = (cofactor.find_determinant());
                         is_negative = true;
                     }
                 }
             }
     
     
  }
  //before returning the matrix, it needs to be transposed
  return (new_matrix.transpose_matrix());
}
/**find_sum(matrix)
 *-RETURNS: new matrix which is formed from finding the sum of two matrices.
 *-PRE: both matrices much have the same dimensions a blank zero matrix is returned 
 *-If dimensions aren't the same, then a blank zero matrix is returned.
 */
Matrix Matrix::find_sum(const Matrix& m) const
{
  int rows = this->get_rows();
  int cols = this->get_cols();
  vector< vector<double> > new_matrix(rows, vector<double>(cols,0));
  vector< vector<double> > temp = this->mat;

  if ((rows == m.get_rows()) && (cols == m.get_cols()))
  {
     for (int r=0; r<rows; ++r)
     {
        for (int c=0; c<cols; ++c)
        {
           new_matrix[r][c] = temp[r][c] + m(r,c);
        }
     }
  }

  return Matrix(new_matrix);
}
/**find_difference(matrix)
 *-RETURNS: new matrix which is formed from finding he difference of two matrices.
 *-PRE: both matrices much have the same dimensions a blank zero matrix is returned 
 *-If dimensions aren't the same, then return a blank zero matrix.
 */
Matrix Matrix::find_difference(const Matrix& m) const
{
  int rows = this->get_rows();
  int cols = this->get_cols();
  vector< vector<double> > new_matrix(rows, vector<double>(cols,0));
  vector< vector<double> > temp = this->mat;

  if ((rows == m.get_rows()) && (cols == m.get_cols()))
  {
     for (int r=0; r<rows; ++r)
     {
        for (int c=0; c<cols; ++c)
        {
           new_matrix[r][c] = temp[r][c] - m(r,c);
        }
     }
  }

  return Matrix(new_matrix);
}

/**transpose_matrix() const
 *-RETURNS: a newly formed matrix which was transformed from the current matrix object
 */
Matrix Matrix::transpose_matrix() const
{
    Matrix original_matrix(this->mat);
    vector< vector<double> > vect_matrix(this->get_cols(), vector<double>(this->get_rows(),0));
    Matrix new_matrix(vect_matrix);
    
    for (int r=0; r<new_matrix.get_rows(); ++r)
    {
      for (int c=0; c<new_matrix.get_cols(); ++c)
     {
     	new_matrix.value(r,c) = original_matrix(c,r);
     }
   }
   return new_matrix;
}
/**value(row,col)
 *-RETURNS: a reference to the matrix's value at the passed indices.  This allows
 *-the class to make calls such as: Matrix.value(r,c) = value; 
 */
double &Matrix::value(int row, int col)
{
    return this->mat[row][col];
}

double Matrix::L2norm()
{
  double sum=0;    
    for (int i=0; i<this->get_rows(); i++)
    {
      for (int j=0; j<this->get_cols(); j++)
     {
      sum += (this->mat[i][j])*(this->mat[i][j]);
     }
   }
   return sqrt(sum);
}


#include <time.h>
#include <stdio.h>
             
/** Hlavny program pocitajuci eigenrumor algormitmus **/
int main(int argc, char* argv[])
{
    double k=0.42; //konstanta pomeru vah e a p matic;
    if (argc>1) k=strtod(argv[1],NULL);
    
    FILE *fp1, *fp2;
    fp1 = fopen("matrix.txt","r");
    fp2 = fopen("ranks.txt","r");
    int rows=0;
    int cols=0;
    int rows_old=0;
    int cols_old=0;
    int using_old = 0;
    
    if (fp1)
    {
      fscanf(fp1,"%i",&rows);
      fscanf(fp1,"%i",&cols);
    }
    
    if (fp2)
    {
      fscanf(fp2,"%i",&rows_old);
      fscanf(fp2,"%i",&cols_old);
    }
    
    /* inicializacia vektorov a,h,r, r_old*/
    
    vector< vector<double> > vect_one(1, vector<double>(rows,1));
    Matrix a1(vect_one);
    Matrix h1(vect_one);
    Matrix a2(vect_one);
    Matrix h2(vect_one);
    Matrix a(vect_one);
    Matrix h(vect_one);
    
    vector< vector<double> > vect_four(1, vector<double>(cols,1));
    vector< vector<double> > vect_five(1, vector<double>(cols,0));
    Matrix r1(vect_four);
    Matrix r1_old(vect_five);
    Matrix r2(vect_four);
    Matrix r2_old(vect_five);
    Matrix r(vect_four);
    Matrix r_old(vect_five);
    /****/
    
    if ((rows_old==rows && cols_old==cols)) 
    {
      if (fp2)
      {
        using_old = 1;
        fseek(fp2, 33, SEEK_CUR);
        for(int i = 0; i < rows; i++)
          for(int j = 0; j < cols; j++)
        {
          double value=0;
          fscanf(fp2, "%lf", &value);
          //cout <<value;
        }
        fseek(fp2, 45, SEEK_CUR);
        for(int i = 0; i < rows; i++)
          for(int j = 0; j < cols; j++)
        {
          double value=0;
          fscanf(fp2, "%lf", &value);
          //cout <<value;
        }
        fseek(fp2, 36, SEEK_CUR);
        //a
        for(int j = 0; j < rows; j++)
        {
          double value=0;
          fscanf(fp2, "%lf", &value);
          a1.value(0,j) = value;
          
        }
        //h
        fseek(fp2, 29, SEEK_CUR);
        for(int j = 0; j < rows; j++)
        {
          double value=0;
          fscanf(fp2, "%lf", &value);
          h1.value(0,j) = value;
          
        }
        //r
        fseek(fp2, 29, SEEK_CUR);
        for(int j = 0; j < cols; j++)
        {
          double value=0;
          fscanf(fp2, "%lf", &value);
          r1_old.value(0,j) = value;
          
        }
      }
    }
        
    
    //cout <<a<<h<<r;
      
    //exit(0);
    
  


    //-Create the first matrix
    vector< vector<double> > vect_two(rows, vector<double>(cols,0));
    if (fp1)
    {
      for(int i = 0; i < rows; i++)
        for(int j = 0; j < cols; j++)
        {
          double value=0;
          fscanf(fp1, "%lf", &value);
          vect_two[i][j] = value;
        }
    }
    Matrix p(vect_two);
    
    vector< vector<double> > vect_three(rows, vector<double>(cols,0));
    vector< vector<double> > vect_six(rows, vector<double>(rows,0));
    Matrix e1(vect_three);
    Matrix e2(vect_three);
    Matrix N(vect_six);
    
    if (fp1)
    {
      for(int i = 0; i < rows; i++)
        for(int j = 0; j < cols; j++)
      {
        double value=0;
        fscanf(fp1, "%lf", &value);
        e1.value(i,j) = value;
      }
      
      for(int i = 0; i < rows; i++)
        for(int j = 0; j < cols; j++)
      {
        double value=0;
        fscanf(fp1, "%lf", &value);
        e2.value(i,j) = value;
      }
    }
    
    fclose(fp1);
    
    int iterations=0;
    while ((r1_old-r1).L2norm()>0.000001)
    {
      iterations++;
      r1_old = r1;
      r1 = p.transpose_matrix()*k*a1.transpose_matrix() + e1.transpose_matrix()*(1-k)*h1.transpose_matrix();
      r1= r1*(1/r1.L2norm());
      a1 = (p*r1).transpose_matrix();
      h1 = (e1*r1).transpose_matrix();
      h1.value(0,rows-1) = 1; 
      r1=r1.transpose_matrix();
      //cout << "Iter" << endl;
    }
    
    /* Negative evalution */
    
    while ((r2_old-r2).L2norm()>0.000001)
    {
      iterations++;
      r2_old = r2;
      r2 = p.transpose_matrix()*k*a2.transpose_matrix() + e2.transpose_matrix()*(1-k)*h2.transpose_matrix();
      r2= r2*(1/r2.L2norm());
      a2 = (p*r2).transpose_matrix();
      h2 = (e2*r2).transpose_matrix();
      h2.value(0,rows-1) = 1; 
      r2=r2.transpose_matrix();
      //cout << "Iter" << endl;
    }
    r = r1 - (r2*0);
    a = a1 - (a2*0);
    h = h1 - (h2*0); 
 
    /* neighbourough matrix */
    N = e1*e1.transpose_matrix();
    for (int i=0; i<rows;i++)
    {
      for (int j=0;j<rows;j++)
      {
        if (N.value(i,i)>0)
        {
          double value=0;
          value = N.value(i,j)/(N.value(i,i)*N.value(j,j));   
          N.value(j,i) = value;
          N.value(i,j) = value;        
        }  
      }
    }
    
    
    cout << rows << endl <<cols<<endl;
    cout << "Provisoning matrix "<< p <<"Evaluation matrix "<< e1 <<endl;
    cout << "A "<< a <<"H "<< h <<endl<< "R "<<r<<endl;
    cout << "N " << N << endl;
    cout << "alpha: "<< k <<endl;
    cout << "Iterations:" << iterations<<endl;
    cout << "Using old vectors: "<< using_old<<endl;
    cout << "alpha: "<< k <<endl;
    
    time_t rawtime;
    struct tm * timeinfo;
    time ( &rawtime );
    timeinfo = localtime ( &rawtime );
    printf ( "%s", asctime (timeinfo) );
    
   //getchar();
}
